using System;
using System.Xml;
using System.Xml.Xsl;
using System.Xml.XPath;

namespace Multiconn.Xml
{
  /// <summary>
  /// XsltContext providing key() extension function.
  /// </summary>
  internal class IndexingXsltContext: XsltContext
  {
    private KeyExtentionFunction keyFuncImpl;

    /// <summary>
    /// Creates new IndexingXsltContext.
    /// </summary>
    /// <param name="manager">Newly created IndexingXsltContext</param>
    public IndexingXsltContext(XPathNavigatorIndexManager manager, XmlNameTable nt) :
      base (nt as NameTable) 
    {      
      keyFuncImpl = new KeyExtentionFunction(manager);
    }
    
    /// <summary>
    /// Not applicable.
    /// </summary>    
    public override int CompareDocument(string baseUri, string nextbaseUri) 
    {
      return 0;
    }

    /// <summary>
    /// Not applicable.
    /// </summary>    
    public override bool PreserveWhitespace(XPathNavigator node) 
    {
      return true;
    }
		
    /// <summary>
    /// Not applicable.
    /// </summary>
    public override bool Whitespace 
    {
      get { return true; }
    }
		
    /// <summary>
    /// No custom variables.
    /// </summary>    
    public override IXsltContextVariable ResolveVariable(string prefix, string name) 
    {
      return null;
    }

    /// <summary>
    /// Resolves key() extension function.
    /// </summary>
    /// <param name="prefix">The prefix of the function as it appears in the XPath expression</param>
    /// <param name="name">The name of the function</param>
    /// <param name="argTypes">An array of argument types for the function being resolved</param>
    /// <returns>KeyExtentionFunction implementation for key() extension function and null
    /// for anything else.</returns>
    public override IXsltContextFunction ResolveFunction(string prefix, string name, 
      XPathResultType[] argTypes) 
    {
      if (prefix == "" && name == "key") 
      {
        if (argTypes.Length != 2)
          throw new ArgumentException("Wrong number of arguments of key() function.");
        else if (argTypes[0] != XPathResultType.String)
          throw new ArgumentException("Type of the first argument of key() function must be string.");
        else
          return keyFuncImpl;
      }
      else 
        return null;
    }
  }
}
