using System;
using System.Xml.Xsl;
using System.Xml.XPath;

namespace Multiconn.Xml
{
	/// <summary>
	/// key() extension function implementation.
	/// </summary>
	internal class KeyExtentionFunction : IXsltContextFunction
	{
    private const int args = 2;
    private static readonly XPathResultType[] argTypes =
      new XPathResultType[] {XPathResultType.String, XPathResultType.Any};
    private XPathNavigatorIndexManager manager;

    /// <summary>
    /// Creates new KeyExtentionFunction object.
    /// </summary>
    /// <param name="manager">Index manager for accessing indexes</param>
		public KeyExtentionFunction(XPathNavigatorIndexManager manager)
		{
			this.manager = manager;
		}

    /// <summary>
    /// Gets the minimum number of arguments for the function.
    /// </summary>
    public int Minargs 
    {
      get { return args; }
    }
		
    /// <summary>
    /// Gets the maximum number of arguments for the function. 
    /// </summary>
    public int Maxargs 
    {
      get { return args; }
    }    
		
    /// <summary>
    /// Gets the supplied XPath types for the function's argument list.
    /// </summary>
    public XPathResultType[] ArgTypes 
    {
      get { return argTypes; }
    }
		
    /// <summary>
    /// Gets the XPathResultType representing the XPath type returned by the function.
    /// </summary>
    public XPathResultType ReturnType 
    {
      get { return XPathResultType.NodeSet; }
    }

		/// <summary>
		/// Provides the method to invoke the function with the given arguments in the given context.
		/// </summary>
		/// <param name="xsltContext">Given XSLT context</param>
		/// <param name="args">Array of actual arguments</param>
		/// <param name="docContext">Context document</param>
		/// <returns></returns>
    public object Invoke(XsltContext xsltContext, object[] args, 
      XPathNavigator docContext) 
    {
      return manager.GetNodes((string)args[0], args[1]);
    }								    
	}
}
