using System;
using System.Collections;
using System.Xml.XPath;
using System.Diagnostics;

namespace Multiconn.Xml
{
	/// <summary>
	/// Index manager. Manages collection of named indexes.
	/// </summary>
	internal class XPathNavigatorIndexManager
	{
		private Hashtable indexes;
    private XPathNavigator nav;
    private bool indexed;

    /// <summary>
    /// Adds new key to the named index.
    /// </summary>
    /// <param name="nav">XPathNavigator over XML document to be indexed</param>
    /// <param name="name">Index name</param>
    /// <param name="key">Key definition</param>
    public void AddKey(XPathNavigator nav, string indexName, KeyDef key) 
    {
      this.indexed = false;
      this.nav = nav;
      //Named indexes are stored in a hashtable.
      if (indexes == null)
        indexes = new Hashtable();
      XPathNavigatorIndex index = (XPathNavigatorIndex)indexes[indexName];
      if (index == null) 
      {
        index = new XPathNavigatorIndex(); 
        indexes.Add(indexName, index);
      }       
      index.AddKey(key);
    }

    /// <summary>
    /// Builds indexes.
    /// </summary>
    public void BuildIndexes() 
    {      
      XPathNavigator doc = nav.Clone();
      //Walk through the all document nodes adding each one matching 
      //'match' expression to the index.
      doc.MoveToRoot();         
      //Select all nodes but namespaces and attributes
      XPathNodeIterator ni = doc.SelectDescendants(XPathNodeType.All, true);
      while (ni.MoveNext()) 
      {
        if (ni.Current.NodeType == XPathNodeType.Element) 
        {
          //Processs namespace nodes
          if (ni.Current.MoveToFirstNamespace()) 
          {
            do 
            {
              foreach (XPathNavigatorIndex index in indexes.Values)
                index.MatchNode(ni.Current);
            }
            while (ni.Current.MoveToNextNamespace());
            ni.Current.MoveToParent();
          }
          //process attributes
          if (ni.Current.MoveToFirstAttribute()) 
          {
            do 
            {
              foreach (XPathNavigatorIndex index in indexes.Values)
                index.MatchNode(ni.Current);
            }
            while (ni.Current.MoveToNextAttribute());
            ni.Current.MoveToParent();
          }
        }
        foreach (XPathNavigatorIndex index in indexes.Values)
          index.MatchNode(ni.Current);
      }            
      indexed = true;
    }
    
    /// <summary>
    /// Get indexed nodes by index name and key value.
    /// </summary>    
    /// <param name="indexName">Index name</param>
    /// <param name="value">Key value</param>
    /// <returns>Indexed nodes</returns>
    public XPathNodeIterator GetNodes(string indexName, object value) 
    {
      if (!indexed)
        BuildIndexes();
      XPathNavigatorIndex index = (XPathNavigatorIndex)indexes[indexName];      
      return index == null? null : index.GetNodes(value);      
    }
	}
}
